@@ -36,8 +36,8 @@
 #include "memalign.h"
 #include "server.h"
 
-unsigned long page_mask;
-unsigned long page_size;
+uintptr_t page_mask;
+uintptr_t page_size;
 
 static int endian_check(void)
 {
@@ -104,6 +104,8 @@ int main(int argc, char *argv[], char *envp[])
 	if (parse_options(argc, argv))
 		return 1;
 
+	fio_time_init();
+
 	if (nr_clients)
 		return fio_handle_clients();
 	else
