@@ -15,8 +15,9 @@
 
 #ifdef ARCH_HAVE_CPU_CLOCK
 static unsigned long cycles_per_usec;
-int tsc_reliable = 0;
+static unsigned long inv_cycles_per_usec;
 #endif
+int tsc_reliable = 0;
 
 struct tv_valid {
 	struct timeval last_tv;
@@ -120,6 +121,15 @@ static void fio_init gtod_init(void)
 
 #endif /* FIO_DEBUG_TIME */
 
+static int fill_clock_gettime(struct timespec *ts)
+{
+#ifdef FIO_HAVE_CLOCK_MONOTONIC
+	return clock_gettime(CLOCK_MONOTONIC, ts);
+#else
+	return clock_gettime(CLOCK_REALTIME, ts);
+#endif
+}
+
 #ifdef FIO_DEBUG_TIME
 void fio_gettime(struct timeval *tp, void *caller)
 #else
@@ -148,11 +158,7 @@ void fio_gettime(struct timeval *tp, void fio_unused *caller)
 	case CS_CGETTIME: {
 		struct timespec ts;
 
-#ifdef FIO_HAVE_CLOCK_MONOTONIC
-		if (clock_gettime(CLOCK_MONOTONIC, &ts) < 0) {
-#else
-		if (clock_gettime(CLOCK_REALTIME, &ts) < 0) {
-#endif
+		if (fill_clock_gettime(&ts) < 0) {
 			log_err("fio: clock_gettime fails\n");
 			assert(0);
 		}
@@ -172,7 +178,7 @@ void fio_gettime(struct timeval *tp, void fio_unused *caller)
 		} else if (tv)
 			tv->last_cycles = t;
 
-		usecs = t / cycles_per_usec;
+		usecs = (t * inv_cycles_per_usec) / 16777216UL;
 		tp->tv_sec = usecs / 1000000;
 		tp->tv_usec = usecs % 1000000;
 		break;
@@ -203,15 +209,22 @@ void fio_gettime(struct timeval *tp, void fio_unused *caller)
 #ifdef ARCH_HAVE_CPU_CLOCK
 static unsigned long get_cycles_per_usec(void)
 {
+	struct timespec ts;
 	struct timeval s, e;
 	unsigned long long c_s, c_e;
 
-	gettimeofday(&s, NULL);
+	fill_clock_gettime(&ts);
+	s.tv_sec = ts.tv_sec;
+	s.tv_usec = ts.tv_nsec / 1000;
+
 	c_s = get_cpu_clock();
 	do {
 		unsigned long long elapsed;
 
-		gettimeofday(&e, NULL);
+		fill_clock_gettime(&ts);
+		e.tv_sec = ts.tv_sec;
+		e.tv_usec = ts.tv_nsec / 1000;
+
 		elapsed = utime_since(&s, &e);
 		if (elapsed >= 1280) {
 			c_e = get_cpu_clock();
@@ -260,11 +273,13 @@ static void calibrate_cpu_clock(void)
 		dprint(FD_TIME, "cycles[%d]=%lu\n", i, cycles[i] / 10);
 
 	avg /= samples;
-	avg = (avg + 9) / 10;
+	avg = (avg + 5) / 10;
 	dprint(FD_TIME, "avg: %lu\n", avg);
 	dprint(FD_TIME, "mean=%f, S=%f\n", mean, S);
 
 	cycles_per_usec = avg;
+	inv_cycles_per_usec = 16777216UL / cycles_per_usec;
+	dprint(FD_TIME, "inv_cycles_per_usec=%lu\n", inv_cycles_per_usec);
 }
 #else
 static void calibrate_cpu_clock(void)
@@ -309,10 +324,10 @@ void fio_clock_init(void)
 		log_info("fio: clocksource=cpu may not be reliable\n");
 }
 
-unsigned long long utime_since(struct timeval *s, struct timeval *e)
+uint64_t utime_since(struct timeval *s, struct timeval *e)
 {
 	long sec, usec;
-	unsigned long long ret;
+	uint64_t ret;
 
 	sec = e->tv_sec - s->tv_sec;
 	usec = e->tv_usec - s->tv_usec;
@@ -332,7 +347,7 @@ unsigned long long utime_since(struct timeval *s, struct timeval *e)
 	return ret;
 }
 
-unsigned long long utime_since_now(struct timeval *s)
+uint64_t utime_since_now(struct timeval *s)
 {
 	struct timeval t;
 
@@ -340,7 +355,7 @@ unsigned long long utime_since_now(struct timeval *s)
 	return utime_since(s, &t);
 }
 
-unsigned long mtime_since(struct timeval *s, struct timeval *e)
+uint64_t mtime_since(struct timeval *s, struct timeval *e)
 {
 	long sec, usec, ret;
 
@@ -361,7 +376,7 @@ unsigned long mtime_since(struct timeval *s, struct timeval *e)
 	return ret;
 }
 
-unsigned long mtime_since_now(struct timeval *s)
+uint64_t mtime_since_now(struct timeval *s)
 {
 	struct timeval t;
 	void *p = __builtin_return_address(0);
@@ -370,7 +385,7 @@ unsigned long mtime_since_now(struct timeval *s)
 	return mtime_since(s, &t);
 }
 
-unsigned long time_since_now(struct timeval *s)
+uint64_t time_since_now(struct timeval *s)
 {
 	return mtime_since_now(s) / 1000;
 }
